/* Copyright 2005 The Apache Software Foundation or its licensors, as
 * applicable.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "apr_strings.h"
#include "ap_config.h"
#include "httpd.h"
#include "http_config.h"
#include "http_log.h"

#include "mod_smtpd.h"

module AP_MODULE_DECLARE_DATA smtpd_load_module;

int smtpd_load_refuse = 0;

smtpd_retcode smtpd_load_connect(smtpd_conn_rec *scr, smtpd_return_data *in)
{
    if (smtpd_load_refuse > 0) {
	int loadavg;
	double avenrun[3];
	
	if (getloadavg(avenrun, sizeof(avenrun) / sizeof(avenrun[0])) < 0) {
	    ap_log_error(APLOG_MARK, APLOG_WARNING, errno, scr->s,
			 "smtpd_load_connect: getloadavg error" );

	    return SMTPD_DECLINED;
	}

	loadavg = (int)(avenrun[0] - 0.5);

	if (loadavg >= smtpd_load_refuse) {
	    char **newstr;
	    in->msgs = apr_array_make(in->p, 1, sizeof(char*));
	    newstr = (char**)apr_array_push(in->msgs);
	    *newstr = apr_pstrdup(in->p, "Please try again later");
	    
	    ap_log_error(APLOG_MARK, APLOG_WARNING, 0, scr->s,
			 "smtpd_load_connect: denying connectin, load %d is => to %d",
			 loadavg, smtpd_load_refuse );

	    return SMTPD_DENYSOFT;
	}
    }

    return SMTPD_DECLINED;
}

static const char *set_smtpd_load_refuse(cmd_parms * cmd, void *config,
					 const char *field)
{
    const char *err = ap_check_cmd_context(cmd, GLOBAL_ONLY);
    if (err != NULL) {
	return err;
    }
    smtpd_load_refuse = atoi( field );
    return NULL;
}

static const command_rec smtpd_load_cmds[] = {

    AP_INIT_TAKE1("SmtpLoadRefuse", set_smtpd_load_refuse, NULL,
		  RSRC_CONF, "load average at which we refuse connections"),
    {NULL}
};

static void register_hooks(apr_pool_t *p)
{
    APR_OPTIONAL_HOOK(smtpd, connect, smtpd_load_connect, NULL, NULL,
		      APR_HOOK_MIDDLE);
}

module AP_MODULE_DECLARE_DATA smtpd_load_module =
{
    STANDARD20_MODULE_STUFF,
    NULL,                          /* dir config creater */
    NULL,                          /* dir merger --- default is to override */
    NULL,                          /* server config */
    NULL,                          /* merge server config */
    smtpd_load_cmds,               /* command apr_table_t */
    register_hooks                 /* register hooks */
};
